--- old/usr/src/uts/common/os/modsysfile.c	Wed Sep  4 04:06:06 2019
+++ new/usr/src/uts/common/os/modsysfile.c	Wed Sep  4 04:06:06 2019
@@ -23,6 +23,7 @@
  * Copyright 2008 Sun Microsystems, Inc.  All rights reserved.
  * Use is subject to license terms.
  * Copyright 2017 Nexenta Systems, Inc.
+ * Copyright 2019 Joyent, Inc.
  */
 
 #include <sys/types.h>
@@ -67,6 +68,11 @@
 char *self_assembly = "/etc/system.d/.self-assembly";
 char *systemfile = "/etc/system";	/* name of ascii system file */
 
+#define	BUILDVERSION_LEN (4096)
+
+char *versionfile = "/etc/versions/build";
+char buildversion[BUILDVERSION_LEN];
+
 static struct sysparam *sysparam_hd;	/* head of parameters list */
 static struct sysparam *sysparam_tl;	/* tail of parameters list */
 static vmem_t *mod_sysfile_arena;	/* parser memory */
@@ -811,6 +817,8 @@
 void
 mod_read_system_file(int ask)
 {
+	struct _buf *file;
+
 	mod_sysfile_arena = vmem_create("mod_sysfile", NULL, 0, 8,
 	    segkmem_alloc, segkmem_free, heap_arena, 0, VM_SLEEP);
 
@@ -838,6 +846,18 @@
 
 	if (ask == 0)
 		setparams();
+
+	/*
+	 * A convenient place to read in our build version string.
+	 */
+
+	if ((file = kobj_open_file(versionfile)) != (struct _buf *)-1) {
+		if (kobj_read_file(file, buildversion,
+		    sizeof (buildversion) - 1, 0) == -1) {
+			cmn_err(CE_WARN, "failed to read %s\n", versionfile);
+		}
+		kobj_close_file(file);
+	}
 }
 
 /*
